unit FontView;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ComCtrls;

type
  TFontViewForm = class(TForm)
    FontBox: TListBox;
    ViewSetup: TGroupBox;
    vmDirect: TRadioButton;
    vmSample: TRadioButton;
    Sample: TEdit;
    BitBtn1: TBitBtn;
    BitBtn2: TBitBtn;
    tbItemSize: TTrackBar;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FontBoxDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure tbItemSizeChange(Sender: TObject);
    procedure vmChange(Sender: TObject);
    procedure SampleChange(Sender: TObject);
    procedure BitBtn2Click(Sender: TObject);
    procedure FormShow(Sender: TObject);
  private
    { Private-Deklarationen}
    ItemSize:integer;
    DefaultFontSize:integer;
    ShowDirect:boolean;
    MaxLenX,MaxLenY:integer;
    fnoX,fnoY:integer;
    SampleText:string;
    procedure AdjustItemSize;
  public
    { Public-Deklarationen}
  end;

var
  FontViewForm: TFontViewForm;

implementation

{$R *.DFM}

procedure TFontViewForm.FormCreate(Sender: TObject);
var i,x,y:integer;
begin
  ShowDirect:=false;
  SampleText:='AaBbCc123';
  ItemSize:=32;

  with FontBox do begin
    Items:=Screen.Fonts;
    MaxLenX:=0;MaxLenY:=0;
    for i:=(Items.Count-1) downto 0 do begin
      x:=Canvas.TextWidth(Items[i]);
      if x>MaxLenX then MaxLenX:=x;
      y:=Canvas.TextHeight(Items[i]);
      if y>MaxLenY then MaxLenY:=y;
    end;
    Canvas.Pen.Style:=psClear;
  end;
  if ShowDirect then vmDirect.Checked:=true else vmSample.Checked:=true;
  Sample.Text:=SampleText;
  Sample.Enabled:=not ShowDirect;
  DefaultFontSize:=Font.Size;
  tbItemSize.Position:=ItemSize;
  AdjustItemSize;
end;

procedure TFontViewForm.FontBoxDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var FontName:string;
begin
  with FontBox,Canvas do begin
    Brush.Style:=bsSolid;
    FillRect(Rect);
    Brush.Style:=bsClear;
    FontName:=Items[Index];
    if ShowDirect then begin
      Font.Name:=FontName;
      Font.Height:=Rect.Bottom-Rect.Top-1;
      TextOut(Rect.Left+1,Rect.Top+1,FontName);
    end else begin
      Font.Name:='Tahoma';
      Font.Size:=DefaultFontSize;
      TextOut(Rect.Left+3,Rect.Top+fnoY,FontName);
      Font.Name:=FontName;
      Font.Height:=ItemSize-2;
      TextOut(Rect.Left+fnoX,Rect.Top+3,SampleText);
    end;
  end;
end;

procedure TFontViewForm.AdjustItemSize;
begin
  fnoX:=MaxLenX+7;
  fnoY:=(ItemSize-MaxLenY) SHR 1;
  FontBox.ItemHeight:=ItemSize;
end;

procedure TFontViewForm.tbItemSizeChange(Sender: TObject);
begin
  ItemSize:=tbItemSize.Position;
  AdjustItemSize;
end;

procedure TFontViewForm.vmChange(Sender: TObject);
begin
  ShowDirect:=vmDirect.Checked;
  Sample.Enabled:=not ShowDirect;
  FontBox.Invalidate;
end;

procedure TFontViewForm.SampleChange(Sender: TObject);
begin
  SampleText:=Sample.Text;
  FontBox.Invalidate;
end;

procedure TFontViewForm.BitBtn2Click(Sender: TObject);
begin
  Tag:=FontBox.ItemIndex;
end;

procedure TFontViewForm.FormShow(Sender: TObject);
begin
  FontBox.ItemIndex:=Tag;
end;

end.
