unit Main;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ExtCtrls,KeyNames, Clipbrd;

const XSpace=20;
      YSpace=24;

type
  TMainForm = class(TForm)
    SelectorPanel: TPanel;
    Selector: TImage;
    FontSel: TComboBox;
    Bevel1: TBevel;
    PreviewPanel: TPanel;
    BSelect: TButton;
    ipCode: TPanel;
    ipKey: TPanel;
    ipChar: TPanel;
    Preview: TLabel;
    ilChar: TLabel;
    BCopyChar: TButton;
    procedure FormCreate(Sender: TObject);
    procedure FontSelChange(Sender: TObject);
    procedure SelectorMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure SelectorMouseDown(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure BSelectClick(Sender: TObject);
    procedure FormKeyPress(Sender: TObject; var Key: Char);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure BCopyCharClick(Sender: TObject);
  private
    { Private-Deklarationen}
    CharX:array[#32..#255]of integer;
    SelectedChar,LastChar:char;
    procedure RedrawCharmap;
    procedure CharChange;
  public
    { Public-Deklarationen}
  end;

var
  MainForm: TMainForm;

implementation

uses FontView;

{$R *.DFM}

procedure TMainForm.FormCreate(Sender: TObject);
begin
  with FontSel do begin
    Items:=Screen.Fonts;
    ItemIndex:=Items.IndexOf('Arial');
  end;
  LastChar:=#32;SelectedChar:=#32;
  RedrawCharmap;
end;

procedure TMainForm.RedrawCharmap;
var x,y,px,py:integer;c:char;
begin
  Preview.Font.Name:=FontSel.Text;
  with Selector.Canvas do begin
    Font.Name:=FontSel.Text;
    Font.Height:=YSpace-3;
    Brush.Style:=bsSolid;
    Brush.Color:=clWindow;
    Pen.Style:=psSolid;
    FillRect(Rect(0,0,32*XSpace+1,7*YSpace+1));
    for x:=0 to 32 do begin
      MoveTo(x*XSpace,0);LineTo(x*XSpace,7*YSpace);
    end;
    for y:=0 to 7 do begin
      MoveTo(0,y*YSpace);LineTo(32*XSpace,y*YSpace);
    end;
    Pen.Style:=psClear;
    Brush.Style:=bsClear;
    Font.Color:=clWindowText;
    for y:=0 to 6 do begin
      py:=y*YSpace+2;
      for x:=0 to 31 do begin
        c:=char((y+1) SHL 5 OR x);
        px:=x*XSpace+(XSpace-TextWidth(c)) SHR 1;
        TextOut(px,py,c);
        CharX[c]:=px;
      end;
    end;
  end;
  CharChange;
end;

procedure TMainForm.FontSelChange(Sender: TObject);
begin
  RedrawCharmap;
end;

procedure TMainForm.SelectorMouseMove(Sender: TObject; Shift: TShiftState;
  X, Y: Integer);
var fx,fy:integer;
begin
  if not (ssLeft in Shift) then exit;
  fx:=X DIV XSpace; if (fx<0) OR (fx>31) then exit;
  fy:=Y DIV YSpacE; if (fy<0) OR (fy>6)  then exit;
  SelectedChar:=char((fy+1) SHL 5 OR fx);
  if SelectedChar<>LastChar then CharChange;
end;

procedure TMainForm.SelectorMouseDown(Sender: TObject;
  Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
  ActiveControl:=SelectorPanel;
  SelectorMouseMove(Sender,Shift,X,Y);
end;

procedure TMainForm.CharChange;
var x,y:integer;
begin
  x:=byte(LastChar) AND 31;y:=(byte(LastChar) SHR 5)-1;
  with Selector.Canvas do begin
    Brush.Style:=bsSolid;
    Brush.Color:=clWindow;
    FillRect(Rect(x*XSpace+1,y*YSpace+1,(x+1)*XSpace,(y+1)*YSpace));
    Brush.Style:=bsClear;
    Font.Color:=clWindowText;
    TextOut(CharX[LastChar],y*YSpace+2,LastChar);
  end;

  Preview.Caption:=SelectedChar;
  x:=byte(SelectedChar) AND 31;y:=(byte(SelectedChar) SHR 5)-1;
  ipCode.Caption:=Format('%03d',[byte(SelectedChar)]);
  ilChar.Caption:=SelectedChar;
  ipKey.Caption:=KeyTab[SelectedChar];

  with Selector.Canvas do begin
    Brush.Style:=bsSolid;
    Brush.Color:=clHighlight;
    FillRect(Rect(x*XSpace+1,y*YSpace+1,(x+1)*XSpace,(y+1)*YSpace));
    Brush.Style:=bsClear;
    Font.Color:=clHighlightText;
    TextOut(CharX[SelectedChar],y*YSpace+2,SelectedChar);
  end;
  LastChar:=SelectedChar;
end;

procedure TMainForm.BSelectClick(Sender: TObject);
var sel:integer;
begin
  FontViewForm.Tag:=FontSel.ItemIndex;
  if (FontViewForm.ShowModal=mrOK) then sel:=FontViewForm.Tag
                                   else sel:=-1;
  if sel<0 then exit;
  FontSel.ItemIndex:=sel;
  RedrawCharmap;
end;

procedure TMainForm.FormKeyPress(Sender: TObject; var Key: Char);
begin
  if ActiveControl=FontSel then exit;
  if Key=^C then Clipboard.AsText:=SelectedChar;
  if (Key<#32) then exit;
  SelectedChar:=Key;
  CharChange;
  Key:=#0;
end;

procedure TMainForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  case Key of
    37:if SelectedChar>#32 then dec(SelectedChar);
    39:if SelectedChar<#255 then inc(SelectedChar);
    38:if SelectedChar>#64 then dec(SelectedChar,32)
                           else SelectedChar:=#32;
    40:if SelectedChar<#223 then inc(SelectedChar,32)
                            else SelectedChar:=#255;
  else exit end;
  CharChange;
  Key:=0;
end;

procedure TMainForm.BCopyCharClick(Sender: TObject);
begin
  Clipboard.AsText:=SelectedChar;
end;

end.
